@@ -173,6 +173,43 @@ abstract class ParserCacheSerializationTestCases {
 		];
 	}
 
+	/**
+	 * This matches PagePropsTable::encodeValue() and represents how
+	 * page properties are actually stored in the database.
+	 * @param mixed $v
+	 * @return int|float|string
+	 */
+	public static function pagePropEncode( $v ) {
+		if ( is_int( $v ) || is_float( $v ) || is_string( $v ) ) {
+			return $v;
+		}
+		if ( is_bool( $v ) ) {
+			return (int)$v;
+		}
+		if ( $v === null ) {
+			return '';
+		}
+		if ( is_array( $v ) ) {
+			// strval on an array yield 'Array' plus a warning.
+			return 'Array';
+		}
+		return strval( $v );
+	}
+
+	/**
+	 * Assert equality in terms of semantically equivalent page property
+	 * values.
+	 * @param MediaWikiIntegrationTestCase $testCase
+	 * @param mixed $expected
+	 * @param mixed $actual
+	 */
+	public static function assertPagePropSame( MediaWikiIntegrationTestCase $testCase, $expected, $actual ): void {
+		$testCase->assertSame(
+			self::pagePropEncode( $expected ),
+			$actual
+		);
+	}
+
 	/**
 	 * Get acceptance test cases for ParserOutput class.
 	 * @see SerializationTestTrait::getTestInstancesAndAssertions()
@@ -209,11 +246,23 @@ abstract class ParserCacheSerializationTestCases {
 			$parserOutputWithProperties->setPageProperty( $key, $value );
 		}
 
+		$parserOutputWithProperties1_45 = new ParserOutput( '' );
+		foreach ( self::MOCK_EXT_DATA as $key => $value ) {
+			$value = self::pagePropEncode( $value );
+			$parserOutputWithProperties1_45->setPageProperty( $key, $value );
+		}
+
 		$parserOutputWithFalsyProperties = new ParserOutput( '' );
 		foreach ( self::MOCK_FALSY_PROPERTIES as $key => $value ) {
 			$parserOutputWithFalsyProperties->setPageProperty( $key, $value );
 		}
 
+		$parserOutputWithFalsyProperties1_45 = new ParserOutput( '' );
+		foreach ( self::MOCK_FALSY_PROPERTIES as $key => $value ) {
+			$value = self::pagePropEncode( $value );
+			$parserOutputWithFalsyProperties1_45->setPageProperty( $key, $value );
+		}
+
 		$parserOutputWithBinaryProperties = new ParserOutput( '' );
 		foreach ( self::MOCK_BINARY_PROPERTIES as $key => $value ) {
 			$parserOutputWithBinaryProperties->setPageProperty( $key, $value );
@@ -264,6 +313,9 @@ abstract class ParserCacheSerializationTestCases {
 			$parserOutputWithMetadata
 		)->mWarningMsgs = [];
 
+		$parserOutputWithMetadataPost1_44 = clone $parserOutputWithMetadata;
+		$parserOutputWithMetadataPost1_44->setLanguageLinks( [ 'm:link1', 'mw:link2' ] );
+
 		$parserOutputWithSections = new ParserOutput( '' );
 		$parserOutputWithSections->setSections( self::SECTIONS );
 
@@ -282,6 +334,9 @@ abstract class ParserCacheSerializationTestCases {
 		$parserOutputWithMetadataPost1_34->addExtraCSPScriptSrc( 'script1' );
 		$parserOutputWithMetadataPost1_34->addLink( Title::makeTitle( NS_SPECIAL, 'Link3' ) );
 
+		$parserOutputWithEmptyToC = new ParserOutput( '' );
+		$parserOutputWithEmptyToC->setSections( [] );
+
 		MWDebug::clearDeprecationFilters();
 
 		$testCases = [
@@ -306,6 +361,7 @@ abstract class ParserCacheSerializationTestCases {
 					$testCase->assertArrayEquals( [], $object->getIndicators() );
 					$testCase->assertSame( '', $object->getTitleText() );
 					$testCase->assertArrayEquals( [], $object->getSections() );
+					$testCase->assertNull( $object->getTOCData() );
 					$testCase->assertArrayEquals( [], $object->getLinks() );
 					$testCase->assertArrayEquals( [], $object->getLinksSpecial() );
 					$testCase->assertArrayEquals( [], $object->getTemplates() );
@@ -408,6 +464,21 @@ abstract class ParserCacheSerializationTestCases {
 					$testCase->assertArrayEquals( self::MOCK_EXT_DATA, $object->getPageProperties() );
 				}
 			],
+			'pageProperties1_45' => [
+				'instance' => $parserOutputWithProperties1_45,
+				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
+					self::assertPagePropSame( $testCase, self::MOCK_EXT_DATA['boolean'], $object->getPageProperty( 'boolean' ) );
+					self::assertPagePropSame( $testCase, self::MOCK_EXT_DATA['null'], $object->getPageProperty( 'null' ) );
+					self::assertPagePropSame( $testCase, self::MOCK_EXT_DATA['number'], $object->getPageProperty( 'number' ) );
+					self::assertPagePropSame( $testCase, self::MOCK_EXT_DATA['string'], $object->getPageProperty( 'string' ) );
+					self::assertPagePropSame( $testCase, self::MOCK_EXT_DATA['array'], $object->getPageProperty( 'array' ) );
+					self::assertPagePropSame( $testCase, self::MOCK_EXT_DATA['map'], $object->getPageProperty( 'map' ) );
+					$testCase->assertArrayEquals(
+						array_map( fn ( $v )=>self::pagePropEncode( $v ), self::MOCK_EXT_DATA ),
+						array_map( fn ( $v )=>self::pagePropEncode( $v ), $object->getPageProperties() )
+					);
+				}
+			],
 			'binaryPageProperties' => [
 				'instance' => $parserOutputWithBinaryProperties,
 				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
@@ -434,6 +505,62 @@ abstract class ParserCacheSerializationTestCases {
 					$testCase->assertArrayEquals( [ 'indicator1' => 'indicator1_value' ], $object->getIndicators() );
 					$testCase->assertSame( 'title_text1', $object->getTitleText() );
 					$testCase->assertArrayEquals( self::SECTIONS, $object->getSections() );
+					$testCase->assertNotNull( $object->getTOCData() );
+					$testCase->assertArrayEquals( [
+						NS_MAIN => [ 'Link1' => 42 ],
+						NS_USER => [ 'Link2' => 43 ]
+					], $object->getLinks() );
+					$testCase->assertArrayEquals( [
+						NS_SPECIAL => [ 'Template1' => 42 ]
+					], $object->getTemplates() );
+					$testCase->assertArrayEquals( [
+						NS_SPECIAL => [ 'Template1' => 4242 ]
+					], $object->getTemplateIds() );
+					$testCase->assertArrayEquals( [ 'Image1' => 1 ], $object->getImages() );
+					$testCase->assertArrayEquals( [ 'Image1' => [
+						'time' => MWTimestamp::convert( TS_MW, 123456789 ), 'sha1' => 'test_sha1'
+					] ], $object->getFileSearchOptions() );
+					$testCase->assertArrayEquals( [ 'https://test.com' => 1 ], $object->getExternalLinks() );
+					$testCase->assertArrayEquals( [ 'tag1' => 'head_item1' ], $object->getHeadItems() );
+					$testCase->assertArrayEquals( [ 'module1' ], $object->getModules() );
+					$testCase->assertArrayEquals( [ 'module_style1' ], $object->getModuleStyles() );
+					$testCase->assertArrayEquals( [ 'key1' => 'value1' ], $object->getJsConfigVars() );
+					$testCase->assertArrayEquals( [ 'warning1' ], $object->getWarnings() );
+					$testCase->assertSame( 'noindex', $object->getIndexPolicy() );
+					$testCase->assertSame( MWTimestamp::convert( TS_MW, 987654321 ), $object->getRevisionTimestamp() );
+					$testCase->assertArrayEquals(
+						[ 'limit_report_key1' => 'value1' ],
+						$object->getLimitReportData()
+					);
+					$testCase->assertArrayEquals(
+						[ 'limit_report_key1' => 'value1' ],
+						$object->getLimitReportJSData()
+					);
+					$testCase->assertTrue( $object->getEnableOOUI() );
+					$testCase->assertTrue( $object->getHideNewSection() );
+					$testCase->assertTrue( $object->getNewSection() );
+					$testCase->assertTrue( $object->getOutputFlag( 'test' ) );
+					$testCase->assertArrayEquals( [ 'test' ], $object->getAllFlags() );
+				}
+			],
+			'withMetadataPost1_44' => [
+				'instance' => $parserOutputWithMetadataPost1_44,
+				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
+					$testCase->assertSame( 42, $object->getSpeculativeRevIdUsed() );
+					$testCase->assertArrayEquals( [ 'm:link1', 'mw:link2' ], $object->getLanguageLinks() );
+					$testCase->assertArrayEquals( [ 'enwiki' => [
+						'interwiki1' => 1,
+						'interwiki2' => 1
+					] ], $object->getInterwikiLinks() );
+					$testCase->assertArrayEquals( [ 'category1', 'category2' ], $object->getCategoryNames() );
+					$testCase->assertArrayEquals( [
+						'category1' => '2',
+						'category2' => '1'
+					], $object->getCategoryMap() );
+					$testCase->assertArrayEquals( [ 'indicator1' => 'indicator1_value' ], $object->getIndicators() );
+					$testCase->assertSame( 'title_text1', $object->getTitleText() );
+					$testCase->assertArrayEquals( self::SECTIONS, $object->getSections() );
+					$testCase->assertNotNull( $object->getTOCData() );
 					$testCase->assertArrayEquals( [
 						NS_MAIN => [ 'Link1' => 42 ],
 						NS_USER => [ 'Link2' => 43 ]
@@ -475,6 +602,7 @@ abstract class ParserCacheSerializationTestCases {
 				'instance' => $parserOutputWithSections,
 				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
 					$testCase->assertArrayEquals( self::SECTIONS, $object->getSections() );
+					$testCase->assertNotNull( $object->getTOCData() );
 				}
 			],
 			'withMetadataPost1_31' => [
@@ -528,6 +656,42 @@ abstract class ParserCacheSerializationTestCases {
 					);
 				}
 			],
+			'withFalsyProperties1_45' => [
+				'instance' => $parserOutputWithFalsyProperties1_45,
+				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
+					self::assertPagePropSame( $testCase,
+						self::MOCK_FALSY_PROPERTIES['boolean'],
+						$object->getPageProperty( 'boolean' )
+					);
+					self::assertPagePropSame( $testCase,
+						self::MOCK_FALSY_PROPERTIES['null'],
+						$object->getPageProperty( 'null' )
+					);
+					self::assertPagePropSame( $testCase,
+						self::MOCK_FALSY_PROPERTIES['number'],
+						$object->getPageProperty( 'number' )
+					);
+					self::assertPagePropSame( $testCase,
+						self::MOCK_FALSY_PROPERTIES['string'],
+						$object->getPageProperty( 'string' )
+					);
+					self::assertPagePropSame( $testCase,
+						self::MOCK_FALSY_PROPERTIES['numstring'],
+						$object->getPageProperty( 'numstring' )
+					);
+					self::assertPagePropSame( $testCase,
+						self::MOCK_FALSY_PROPERTIES['array'],
+						$object->getPageProperty( 'array' )
+					);
+				}
+			],
+			'withEmptyToC' => [
+				'instance' => $parserOutputWithEmptyToC,
+				'assertions' => static function ( MediaWikiIntegrationTestCase $testCase, ParserOutput $object ) {
+					$testCase->assertArrayEquals( [], $object->getSections() );
+					$testCase->assertNotNull( $object->getTOCData() );
+				},
+			],
 		];
 		// We don't serialize or restore parseStartTime any more, so
 		// ensure that it is cleared in the instances we are going to
